rem
rem Script:   frequency_histogram_edge_case.sql
rem Author:   Randolf Geist
rem Dated:    April 2010
rem Update:   April 2012
rem Purpose:  Edge case of single bucket frequency histograms
rem
rem Last tested
rem   11.1.0.7
rem   11.2.0.1
rem   10.2.0.4
rem
rem The change introduced how to treat non-existing values
rem with frequency histograms shows an interesting anomaly
rem when dealing with a very popular value that results in
rem a single bucket histogram
rem
rem This results in an odd overestimate of non-existing values
rem Note that this applies in general to all cases where
rem the least popular value of the frequency histogram
rem is still very popular
rem
rem In 10g the problem of the single bucket anomaly
rem doesn't appear by default because it produces
rem consistent results for histogram and basic
rem column statistics
rem
rem Forcing the same inconsistency by a PL/SQL block like this
set doc off
doc
declare
 srec dbms_stats.statrec;
 novals dbms_stats.chararray;
 distcnt number;
 density number;
 nullcnt number;
 avgclen number;
begin
  sys.dbms_stats.get_column_stats(
  ownname=>user,
  tabname=>'T1',
  colname=>'T_STATUS',
  distcnt => distcnt,
  density=>density,
  nullcnt=>nullcnt,
  srec=>srec,
  avgclen=>avgclen
  );
  dbms_stats.set_column_stats(
  ownname=>user,
  tabname=>'T1',
  colname=>'T_STATUS',
  distcnt=>3,
  nullcnt=>nullcnt,
  srec=>srec,
  avgclen=>avgclen,
  flags=>2
  );
end;
/
#
rem shows the same behaviour in 10g. So it is definitely a problem
rem caused by the inconsistency of the basic column statistics
rem and the histogram

set echo on linesize 130 pagesize 9999 serveroutput off trimspool on

spool frequency_histogram_edge_case.log

alter session set nls_language = 'AMERICAN';

column column_name format a15

drop table t1;

purge table t1;

create table t1 (
  id number(*, 0)
, t_status number(*, 0)
, t_other_col number(*, 0)
, vc varchar2(100)
);

-- 1 million rows
-- One very popular value
-- Two very unpopular values
-- in column T_STATUS
insert /*+ append */ into t1 (id, t_status, t_other_col, vc)
with generator as (
select  /*+ materialize */
        level as id
from
        dual
connect by
        level <= 10000
)
select  /*+ use_nl(v1, v2) */
        rownum as id
      , case
        when rownum <= 10
        then 1
        when rownum <= 20
        then 2
        else 0
        end as t_status
      , (mod(rownum, 2) + 1) * 100 as t_other_col
      , rpad('x', 100) as vc
from
        generator v1
      , generator v2
where
        rownum <= 1000000;

commit;

create index t1_idx on t1 (t_status);

create index t1_idx2 on t1 (t_other_col);

-- If you run this GATHER_TABLE_STATS using sampling often enough, you'll eventually pick up one of the unpopular value in T_STATUS
-- Then the results below will be completely different => Non-deterministic, inconsistent behaviour
exec dbms_stats.gather_table_stats(null, 'T1', method_opt => 'for all columns size 1 for columns t_status size 254, t_other_col size 2')

-- This is the real data distribution. If you compute the Frequency Histogram it will correspond to this result
-- A Frequency Histogram is simply VALUE + COUNT(*) of VALUE
select
        t_status
      , count(*) as cnt
from
        t1
group by
        t_status
order by
        cnt desc
;

-- This is the real data distribution. If you compute the Frequency Histogram it will correspond to this result
-- A Frequency Histogram is simply VALUE + COUNT(*) of VALUE
select
        t_other_col
      , count(*) as cnt
from
        t1
group by
        t_other_col
order by
        cnt desc
;

-- Note that the basic column statistics
-- are generated with a much higher sample size
-- The histogram however was created with a sample size
-- of 5,000 rows only
-- Therefore we get three distinct values but only a single bucket
select
       column_name
     , num_distinct
     , num_buckets
     , sample_size
     , histogram
from
       user_tab_col_statistics
where
       table_name = 'T1';

-- The low default sample size usually results in a single bucket
-- But not always => Potential instability issue
select
        column_name
      , endpoint_number
      , endpoint_value
from
        user_tab_histograms
where
        table_name = 'T1'
and     column_name in ('T_STATUS', 'T_OTHER_COL')
order by
        column_name
      , endpoint_number;

-- T_OTHER_COL

-- The popular value
explain plan for
select
        *
from
        t1
where
        t_other_col = 100
;

select * from table(dbms_xplan.display);

-- A non-existing, within range value
explain plan for
select
        *
from
        t1
where
        t_other_col = 150
;

select * from table(dbms_xplan.display);

-- Two non-existing values
explain plan for
select
        *
from
        t1
where
        t_other_col in (125, 175)
;

select * from table(dbms_xplan.display);

-- Two non-existing values, out of range
explain plan for
select
        *
from
        t1
where
        t_other_col in (1000, 2000)
;

select * from table(dbms_xplan.display);

-- T_STATUS

-- The popular value
explain plan for
select
        *
from
        t1
where
        t_status = 0
;

select * from table(dbms_xplan.display);

-- A non-existing value
explain plan for
select
        *
from
        t1
where
        t_status = 1000
;

select * from table(dbms_xplan.display);

-- Two non-existing values
explain plan for
select
        *
from
        t1
where
        t_status in (1000, 2000)
;

select * from table(dbms_xplan.display);

-- A very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status = 1
;

select * from table(dbms_xplan.display);

-- Another very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status = 2
;

select * from table(dbms_xplan.display);

-- One non-existing values, Dynamic Sampling using index
explain plan for
select  /*+ dynamic_sampling(t1, 2) dynamic_sampling_est_cdn(t1) */
        *
from
        t1
where
        t_status = 1000
;

select * from table(dbms_xplan.display);

-- Two non-existing values, Dynamic Sampling fails to generate plan that uses the index (bug)
explain plan for
select  /*+ dynamic_sampling(t1, 2) dynamic_sampling_est_cdn(t1) */
        *
from
        t1
where
        t_status in (1000, 2000)
;

select * from table(dbms_xplan.display);

-- Switch off new behaviour
alter session set "_fix_control" = '5483301:off';

-- T_OTHER_COL

-- The popular value
explain plan for
select
        *
from
        t1
where
        t_other_col = 100
;

select * from table(dbms_xplan.display);

-- A non-existing, within range value
explain plan for
select
        *
from
        t1
where
        t_other_col = 150
;

select * from table(dbms_xplan.display);

-- Two non-existing values
explain plan for
select
        *
from
        t1
where
        t_other_col in (125, 175)
;

select * from table(dbms_xplan.display);

-- T_STATUS

-- The popular value
explain plan for
select
        *
from
        t1
where
        t_status = 0
;

select * from table(dbms_xplan.display);

-- A non-existing value
explain plan for
select
        *
from
        t1
where
        t_status = 1000
;

select * from table(dbms_xplan.display);

-- Two non-existing values
explain plan for
select
        *
from
        t1
where
        t_status in (1000, 2000)
;

select * from table(dbms_xplan.display);

-- A very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status = 1
;

select * from table(dbms_xplan.display);

-- Another very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status = 2
;

select * from table(dbms_xplan.display);

-- Gather statistics with 100% sample size
exec dbms_stats.gather_table_stats(null, 'T1', estimate_percent => null, method_opt => 'for all columns size 1 for columns t_status size 254, t_other_col size 2')

-- Now the statistics are more representative
select
       column_name
     , num_distinct
     , num_buckets
     , sample_size
     , histogram
from
       user_tab_col_statistics
where
       table_name = 'T1';

-- The histogram now covers also the unpopular values
select
        column_name
      , endpoint_number
      , endpoint_value
from
        user_tab_histograms
where
        table_name = 'T1'
and     column_name in ('T_STATUS', 'T_OTHER_COL')
order by
        column_name
      , endpoint_number;

-- Switch on new behaviour
alter session set "_fix_control" = '5483301:on';

-- T_OTHER_COL

-- The popular value
explain plan for
select
        *
from
        t1
where
        t_other_col = 100
;

select * from table(dbms_xplan.display);

-- A non-existing, within range value
explain plan for
select
        *
from
        t1
where
        t_other_col = 150
;

select * from table(dbms_xplan.display);

-- Two non-existing values
explain plan for
select
        *
from
        t1
where
        t_other_col in (125, 175)
;

select * from table(dbms_xplan.display);

-- T_STATUS

-- A non-existing value
explain plan for
select
        *
from
        t1
where
        t_status = 1000
;

select * from table(dbms_xplan.display);

-- Two non-existing values
explain plan for
select
        *
from
        t1
where
        t_status in (1000, 2000)
;

select * from table(dbms_xplan.display);

-- A very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status = 1
;

select * from table(dbms_xplan.display);

-- Another very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status = 2
;

select * from table(dbms_xplan.display);

-- Add a virtual column (the same could be achieved using a function-based index instead for pre-11g versions)
alter table t1 add (t_status_unpop as (case when t_status != 0 then t_status end));

exec dbms_stats.gather_table_stats(null, 'T1', method_opt => 'for columns t_status_unpop size 254')

-- Since the virtual column only covers
-- the unpopular values, the histogram
-- will be very precise even with a low sample size
select
       column_name
     , num_distinct
     , sample_size
from
       user_tab_col_statistics
where
       table_name = 'T1';

-- The histogram only covers the unpopular values
select
        column_name
      , endpoint_number
      , endpoint_value
from
        user_tab_histograms
where
        table_name = 'T1'
and     column_name = 'T_STATUS_UNPOP';

-- Two non-existing values
-- So we don't run into the same problem here
-- The estimate is reasonable for non-existing values
explain plan for
select
        *
from
        t1
where
        t_status_unpop in (1000, 2000)
;

select * from table(dbms_xplan.display);

-- A very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status_unpop = 1
;

select * from table(dbms_xplan.display);

-- Another very unpopular value
explain plan for
select
        *
from
        t1
where
        t_status_unpop = 2
;

select * from table(dbms_xplan.display);

spool off
